<?php
/**
 * @copyright Copyright (c) 2021 Aurora Creation Sp. z o.o. (http://auroracreation.com)
 */

declare(strict_types=1);

namespace Aurora\Santander\Service;

use Aurora\Santander\Helper\Config;
use Magento\Sales\Api\OrderRepositoryInterface;
use SoapFault;
use SimpleXMLElement;
use Magento\Setup\Exception;
use Symfony\Component\Console\Output\OutputInterface;
use Magento\Sales\Model\ResourceModel\Order\Collection;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory as OrderCollectionFactory;

class ProcessOrders
{
    const SANTANDER_URL = 'https://api.santanderconsumer.pl/ProposalServiceHybrid';
    const SANTANDER_PAYMENT_METHOD = 'eraty_santander';
    const SANTANDER_CLOSED_STATES = ['Odmowa', 'Wydaj_towar'];

    /**
     * @var OrderCollectionFactory
     */
    private $orderCollectionFactory;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @param OrderCollectionFactory $orderCollectionFactory
     * @param OrderRepositoryInterface $orderRepository
     * @param Config $config
     */
    public function __construct(
        OrderCollectionFactory   $orderCollectionFactory,
        OrderRepositoryInterface $orderRepository,
        Config                   $config
    )
    {
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->config = $config;
        $this->orderRepository = $orderRepository;
    }

    /**
     * @return Collection
     */
    private function getOrderCollectionWithSantanderPaymentMethod()
    {
        $collection = $this->orderCollectionFactory->create();
        $collection->getSelect()->join(
            ['sop' => 'sales_order_payment'], 'main_table.entity_id = sop.parent_id', array('method')
        )->where('sop.method = ?', self::SANTANDER_PAYMENT_METHOD);
        $collection->getSelect()->where('not santander_bank_status_code = ?', StatusCode::CLOSED);

        return $collection;
    }

    /**
     * @param $santanderId
     * @return array
     */
    private function getOrderStateById($santanderId)
    {
        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_URL => self::SANTANDER_URL,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_SSLCERT => $this->config->getCertPath(),
            CURLOPT_SSLCERTPASSWD => $this->config->getCertPasswd(),
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => sprintf(
                '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
                                          xmlns:san="http://santanderconsumerbank.pl/" 
                                          xmlns:obj="http://santanderconsumerbank.pl/objects" 
                                          xmlns:arr="http://schemas.microsoft.com/2003/10/Serialization/Arrays"
                        >
                        <soapenv:Header/>
                        <soapenv:Body>
                            <san:GetApplicationState>
                                <san:Identity>
                                    <obj:Login>%s</obj:Login>
                                    <obj:Password>%s</obj:Password>
                                    <obj:ShopNumber>%s</obj:ShopNumber>
                                </san:Identity>
                                <san:ShopApplicationNumbers>
                                    <arr:string>%s</arr:string>
                                </san:ShopApplicationNumbers>
                            </san:GetApplicationState>
                        </soapenv:Body>
                    </soapenv:Envelope>',
                $this->config->getLogin(), $this->config->getPassword(), $this->config->getShopNumber(), $santanderId),
            CURLOPT_HTTPHEADER => [
                'Content-Type: text/xml; charset=utf-8',
                'SOAPAction: http://santanderconsumerbank.pl/IProposalServiceHybrid/GetApplicationState'
            ],
        ]);

        $response = curl_exec($curl);
        $response = preg_replace("/(<\/?)(\w+):([^>]*>)/", "$1$2$3", $response);
        $xml = new SimpleXMLElement($response);
        $body = $xml->xpath('//sBody')[0];

        return json_decode(json_encode((array)$body), TRUE);
    }

    /**
     * @param OutputInterface|null $output
     * @return void
     * @throws SoapFault|Exception
     */
    public function updateStatusAll(?OutputInterface $output = null)
    {
        $orders = $this->getOrderCollectionWithSantanderPaymentMethod();

        foreach ($orders as $order) {
            $id = $order->getId();
            $response = $this->getOrderStateById(sprintf('%09d', $id));

            if (!isset($response['GetApplicationStateResponse']['GetApplicationStateResult']['aApplications']['aApplicationData']['aCreditState'])) {
                if ($output) {
                    $output->writeln(sprintf('<error>[%s] [Order ID: %s] response does not contain correct body...</error>', $this->config->getPrefix(), $id));
                }
                continue;
            }

            $status = $response['GetApplicationStateResponse']['GetApplicationStateResult']['aApplications']['aApplicationData']['aCreditState'];
            $order->setSantanderBankStatusCode(
                $status == array_contains(self::SANTANDER_CLOSED_STATES, $status)
                    ? StatusCode::CLOSED : StatusCode::ENABLE
            );
            $order->setSantanderBankResponseStatus($status);
            $this->orderRepository->save($order);

            if ($output) {
                $output->writeln(sprintf('<info>[%s] [Order ID: %s] Order updated correctly...</info>', $this->config->getPrefix(), $id));
            }
        }
    }
}
